<?php

function db_connect()
{
  global $cfg;

  $dsn = "dbname=".$cfg['dbname'];
  if (isset($cfg['dbhost'])) $dsn .= " host=".$cfg['dbhost'];
  if (isset($cfg['dbport'])) $dsn .= " port=".$cfg['dbport'];
  if (isset($cfg['dbuser'])) $dsn .= " user=".$cfg['dbuser'];
  if (isset($cfg['dbpass'])) $dsn .= " password=".$cfg['dbpass'];

  $dbh = pg_pconnect ($dsn)
    or carp("Unable to connect to SQL server");
  pg_exec ($dbh, "SET DateStyle='ISO'");

  return $dbh;
}

function db_query($query)
{
  global $cfg;

  // error_log($query);
  $sth = pg_exec ($cfg['dbh'], $query);

  if ($sth === false) {
    error_log ($query);
    error_log (pg_last_error($cfg['dbh']));
    return false;
  }

  return $sth;
}

function db_last_id($table, $column)
{
  $query = sprintf ("SELECT currval('%s_%s_seq')", $table, $column);

  $sth =  db_query ($query);

  if (!$sth)
    return false;

  if (pg_num_rows($sth) == 0) {
    error_log ("$query resulted in an empty set");
    return false;
  }

  $row = pg_fetch_row($sth, 0);

  return $row[0];
}

// preserve <p></p>, <b></b>, <em></em>, <br>, <a href></a>
function string_sanitise($text)
{
  $ntext = preg_replace (array('/<a\s+href=["\']?([^>"\']*)["\']?>([^<]*)<\/a>/i'),
			 array('[[$1][$2]]'),
			 $text);

  $ntext = htmlspecialchars($ntext, ENT_QUOTES);

  $ntext = preg_replace(array('/&lt;(\/?(p|br|b|em))&gt;/i',
			      '/\[\[([^\]]+)\]\[([^\]]+)\]\]/'),
			array('<$1>','<a href="$1">$2</a>'),
			$ntext);

  return $ntext;
}

function basepath()
{
  global $cfg;
  global $_SERVER;

  $pos = strpos(strtolower($cfg['home']), strtolower($_SERVER['SERVER_NAME']));
  $base = substr($cfg['home'], $pos+strlen($_SERVER['SERVER_NAME']));

  $pos = strpos($_SERVER['REQUEST_URI'], "?");
  if ($pos === false)
    $uri = $_SERVER['REQUEST_URI'];
  else 
    $uri = substr($_SERVER['REQUEST_URI'], 0, $pos);

  if (strpos($uri, $base) === 0) {
    $cfg['path'] = substr($uri, strlen($base));
    $pos = strrpos($cfg['path'], '/');
    if ($pos !== false) {
      $cfg['dir'] = substr($cfg['path'], 0, $pos);
    }
    return str_repeat("../", substr_count($cfg['path'], "/"));
  }
}

function logged_in()
{
  global $_SESSION;

  return isset($_SESSION['uid']);
}

function javascript_ok()
{
  global $_SESSION;

  if (!logged_in())
    return true;

  if ($_SESSION['javascript'] == true)
    return true;

  return false;
}

function is_spider()
{
  global $_SERVER;
  global $_SESSION;

  if (isset($_SESSION['uid']))
    if (isset($_SESSION['robot']))
      return $_SESSION['robot'];

  if (strpos($_SERVER['HTTP_USER_AGENT'], 'Yahoo! Slurp') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Googlebot') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Mediapartners-Google') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'VoilaBot') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Gigabot/3.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Speedy Spider') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'LinkWalker/2.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'proximic') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Yeti/1.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Eurobot/1.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'MnoGoSearch/') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'ia_archiver') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Seekbot/1.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'MyEngines-Bot') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'larbin_') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'findlinks/1') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'holmes/3.12') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'NoteworthyBot/0.1') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Eurosoft-Bot') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Eurobot/1') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Yandex') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'msnbot') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Twiceler-0.9') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'ScoutJet') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'penthesilea/0.3') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'VisBot/2.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Netluchs/Nutch-1.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Yanga WorldSearch Bot') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Semager') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'DotBot') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Baiduspider') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'MJ12bot/') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Yahoo-MMCrawler') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Charlotte/1.1') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Gaisbot/3.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'psbot/0.1') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Ask Jeeves/Teoma') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'SapphireWebCrawler/Nutch') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'kalooga/KaloogaBot') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'SimilarPages/Nutch-1.0-dev') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Cogentbot/1.') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'CatchBot/1.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'GingerCrawler/1.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'yacybot') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'www.yacy.net') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Nutch-1.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Tagoobot/3.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'SapphireWebCrawler/1.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'BotOnParade') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'SurveyBot/2.3') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'XmarksFetch/1.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'spbot/2.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'TinEye/1.1') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'TurnitinBot/2.1') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'CligooRobot/1.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'libwww-perl') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Wget') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Python-urllib') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'CamontSpider/1.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'TwengaBot-Discover') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Hailoobot/1.2') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'bingbot/2.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Exabot/3.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'spbot/2.1') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Search17Bot/1') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Linguee Bot') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'findlinks/2') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'SiteBot/0.1') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Purebot/1.1') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Ezooms/1.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'discobot/1.1') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'AhrefsBot/1.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'suggybot v0.01a') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'DomainCrawler/2.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Plukkie/1') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], '/Nutch-') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'aiHitBot/1.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'AhrefsBot/') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Crawler') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Acoon') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'aiHitBot') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'SeznamBot') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'CareerBot/1') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'SiteExplorer/1') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'discoverybot/2.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'SemrushBot/') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'archive.org_bot') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'waybackarchive.org/1.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'spbot/4.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Mail.RU_Bot/2.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'linkdexbot/2.0') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'webmeup-crawler.com') !== false ||
      strpos($_SERVER['HTTP_USER_AGENT'], 'Cityreview Robot') !== false) {
    if (isset($_SESSION['uid']))
      $_SESSION['robot'] = true;
    return true;
  }

  if (isset($_SESSION['uid']))
    $_SESSION['robot'] = false;
  return false;
}

function is_admin()
{
  global $_SESSION;

  if (!array_key_exists('nickname', $_SESSION))
    return false;

  error_log( $_SESSION['nickname'] === 'Joey');
  return $_SESSION['nickname'] === 'Joey';
}

function format_info_bookmarks()
{
  global $_SESSION;
  global $cfg;

  if (!isset($_SESSION['uid']))
    return false;

  $query = sprintf("SELECT zeitung,name FROM bookmarks " .
		   "JOIN zeitungen ON zeitungen.id = zeitung " .
		   "WHERE uid = %d " .
		   "ORDER BY priority,name LIMIT 20",
		   $_SESSION['uid']);

  $sth = db_query($query);

  if ($sth === false || pg_num_rows ($sth) == 0)
    return false;

  $ret = '<p><ul>';
  for ($n=0; $n < pg_num_rows ($sth); $n++) {
    $row = pg_fetch_array ($sth, $n);
    $ret .= sprintf('<li><a href="%szeitung/%d.html">%s</a></li>',
		    $cfg['basepath'], $row['zeitung'], $row['name']);
  }
  $ret .= '</ul></p>';

  return $ret;
}

?>
